// Copyright (c) 2026 Nikolay Govorov
// SPDX-License-Identifier: AGPL-3.0-or-later

import "@/index.css";
import { mountPage } from "@/lib/mount";
import { Page } from "@/components/pages/dashboard";

mountPage(Page);
